// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@infoscreen.com, or by
// paper mail at:
//
// Tim Freeman
// 655 S. FairOaks Ave., Apt B-316
// Sunnyvale, CA 94086
//

#ifndef __InferBondsConfiguration_h__
#include "InferBondsConfiguration.h"
#endif

#ifndef __String_h__
#include "String.h"
#endif

#ifndef __MemoryUtil_h__
#include "MemoryUtil.h"
#endif

#ifndef __PositiveFloatSlotValue_h__
#include "PositiveFloatSlotValue.h"
#endif

namespace {
  String maxLengthName () {
    return "Maximum bond length";
  }
}

InferBondsConfiguration::InferBondsConfiguration () {
  initializeSlot (maxLengthName (), NEW (PositiveFloatSlotValue (1.7)));
}

Float InferBondsConfiguration::getMaxLength () const {
  return PositiveFloatSlotValue::getSlot (this, maxLengthName ());
}

void InferBondsConfiguration::setMaxLength (Float length) {
  PositiveFloatSlotValue::setSlot (this, maxLengthName (), length);
}

SP<Configuration> InferBondsConfiguration::copy () const {
  return NEW (InferBondsConfiguration (*this));
}

bool InferBondsConfiguration::hasTheSameTypeAs (const Configuration *c)
  const
{
  return 0 != dynamic_cast <const InferBondsConfiguration *> (c);
}
